@@ -17,6 +17,11 @@ class SGraph:
     def directed(self):
         return nx.is_directed(self.get_nx_graph())
 
+    def to_undirected(self):
+        if self.directed():
+            return SGraph(self.gr.to_undirected())
+        return SGraph(self.gr.copy())
+
     def number_of_edges(self):
         return self.gr.number_of_edges()
 
