@@ -1,30 +1,79 @@
 import numpy as np
 import networkx as nx
 
-from scipy.stats import powerlaw
-
+from dtg.tail.estimate.hill import HillEstimator
+from dtg.tail.mse import boot_estimate
 from pge.init.classes.graph import SGraph
 from pge.init.add import actual
 
 
+def powerlaw(n, alp, st=1, ac=0.3, status=None, status_back=False):
+    if status is None:
+        alp_ = alp + 0.5
+    else:
+        alp_ = status
+    while True:
+        rs = np.trunc(nx.utils.powerlaw_sequence(n, alp_)) - 1 + st
+        alp2 = boot_estimate(
+            HillEstimator,
+            rs,
+            1 / 2,
+            2 / 3,
+            50,
+            speed=False
+        )[0]
+        if np.abs(alp2-alp) <= ac:
+            break
+        else:
+            if alp2 > alp:
+                alp_ -= 0.15
+            else:
+                alp_ += 0.15
+    if status_back:
+        return rs, alp_
+    return rs
+
+
+def powerlaw_degree(n, alpha, beta, ac=0.3, status=(None, None), rep=10):
+    out_d, out_status = powerlaw(n, beta, 1, ac=ac, status_back=True, status=status[0])
+    ind_d, in_status = powerlaw(n, alpha, 1, ac=ac, status_back=True, status=status[1])
+
+    zv = np.sum(np.subtract(out_d, ind_d))
+    for _ in np.arange(rep):
+        out_d_, out_status = powerlaw(n, beta, int((beta > alpha))*(np.abs(zv)//n) + 1, ac=ac,
+                                      status_back=True, status=out_status)
+        if np.abs(np.sum(np.subtract(out_d_, ind_d))) < np.abs(zv):
+            out_d = out_d_
+            zv = np.sum(np.subtract(out_d, ind_d))
+        ind_d_, in_status = powerlaw(n, alpha, int((beta < alpha))*(np.abs(zv)//n) + 1, ac=ac,
+                                     status_back=True, status=in_status)
+        if np.abs(np.sum(np.subtract(out_d, ind_d_))) < np.abs(zv):
+            ind_d = ind_d_
+            zv = np.sum(np.subtract(out_d, ind_d))
+
+    print(np.sum(np.subtract(out_d, ind_d)))
+
+    return ind_d, out_d, (in_status, out_status)
+
+
 class Creator:
     @staticmethod
-    def tbt(nm, alpha, beta):
-        ind_d = np.around(1/powerlaw.rvs(alpha, size=nm))
-        out_d = np.around(1/powerlaw.rvs(beta, size=nm))
-
-        zv = np.sum(np.subtract(ind_d, out_d))
+    def tbt(nm, alpha, beta, prm=0.7):
         k = 1 - min([0.5, 1 - alpha ** -1, 1 - beta ** -1]) / 2
-        while np.abs(zv) > nm ** k:
-            i_s = np.random.choice(nm, min(int(np.abs(zv)), nm), replace=False)
+        zv = nm
 
-            for i in i_s:
-                if zv < 0:
-                    ind_d[i] = ind_d[i] + 1
-                else:
-                    out_d[i] = out_d[i] + 1
+        status = (None, None)
+        while np.abs(zv) > nm ** (1 - k + prm * k):
+            ind_d, out_d, status = powerlaw_degree(nm, alpha, beta, status=status)
             zv = np.sum(np.subtract(ind_d, out_d))
 
+        i_s = np.random.choice(nm, min(int(np.abs(zv)), nm), replace=False)
+        for i in i_s:
+            if zv < 0:
+                ind_d[i] = ind_d[i] + 1
+            else:
+                out_d[i] = out_d[i] + 1
+
         gr = SGraph(nx.DiGraph())
         gr.add_nodes(np.arange(nm))
 
@@ -37,6 +86,8 @@ class Creator:
             for j in i_s:
                 if out_d[int(j)] > gr.count_out_degree(j):
                     gr.add_edge(j, i)
+                if ind_d[int(j)] > gr.count_in_degree(j):
+                    gr.add_edge(i, j)
 
         return gr
 
@@ -157,7 +208,7 @@ class Creator:
 
     @staticmethod
     def power_law(n, sigma):
-        inds = np.floor(1/powerlaw.rvs(sigma, size=n))
+        inds = powerlaw(n, sigma)
         if np.sum(inds) % 2 == 1:
             inds = np.append(inds, [1])
 
