@@ -17,7 +17,7 @@ class PADiGrowth(BDiGrowth):
         if tp == "in":
             probs = np.array([graph.count_in_degree(node) for node in graph.get_ids()]) + self.param[0]
         else:
-            probs = np.array([graph.count_in_degree(node) for node in graph.get_ids()]) + self.param[1]
+            probs = np.array([graph.count_out_degree(node) for node in graph.get_ids()]) + self.param[1]
         probs = probs / np.sum(probs)
         return np.random.choice(graph.get_ids(), sz, replace=False, p=probs)
 
